#ifndef KRIMI_DINNER_ENGINE_RENDERER_H
#define KRIMI_DINNER_ENGINE_RENDERER_H

#include <GLES3/gl3.h>
#include <unordered_map>
#include <vector>

#include "Texture.h"
#include "StringRepository.h"

class Renderer
{
public:
    static Renderer* ptr;

    static void create();

    static void destroy();

    void addRect(float x, float y, float w, float h);

    void addRect(float x, float y, float w, float h, StringHandle texture);

    void addRect(float x, float y, float w, float h, float r, float g, float b, float a);

    void addRect(float x, float y, float w, float h, float r, float g, float b, float a, StringHandle texture);

    void addRect(float x, float y, float w, float h, StringHandle texture, float src_x, float src_y, float src_w, float src_h);

    void addRect(float x, float y, float w, float h, float r, float g, float b, float a, StringHandle texture, float src_x, float src_y, float src_w, float src_h);

    void renderFrame(float width, float height);

private:
    Renderer();
    ~Renderer();

    Renderer(const Renderer&) = delete;
    Renderer& operator=(const Renderer&) = delete;

    struct Rect {
        float dst_p0_x, dst_p0_y;
        float dst_p1_x, dst_p1_y;
        float src_p0_x, src_p0_y;
        float src_p1_x, src_p1_y;
        float r, g, b, a;
    };

    GLuint m_shader;
    GLuint m_vao;
    GLuint m_vbo;

    StringHandle m_dummy_texture;

    std::vector<Rect> m_rects;
    std::vector<StringHandle> m_draw_textures;

    std::unordered_map<StringHandle, Texture> m_textures;
};


#endif